/*
 * Copyright (c) 2018, Intel Corporation
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of Intel Corporation nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef SRC_IXP_TYPES_H_
#define SRC_IXP_TYPES_H_

#ifdef __cplusplus
extern "C"
{
#endif


enum ixp_status
{
	IXP_SUCCESS,
	IXP_NOT_YET_IMPLEMENTED,
	IXP_NULL_INPUT_PARAM,
	IXP_NO_MEM_RESOURCES,
	IXP_INTERNAL_ERROR,
	IXP_INVALID_PROP_KEY
};

// NOTE: For simplicity in ixp_fis_properties.c, put fis properties first
//       and contiguously.
enum ixp_prop_key
{
	FIS_IDENTIFY_DIMM_VENDOR_ID,
	FIS_IDENTIFY_DIMM_DEVICE_ID,
	FIS_IDENTIFY_DIMM_REVISION_ID,
	FIS_IDENTIFY_DIMM_INTERFACE_FORMAT_CODE,
	FIS_IDENTIFY_DIMM_FIRMWARE_REVISION,
	FIS_IDENTIFY_DIMM_RESERVED_OLD_API,
	FIS_IDENTIFY_DIMM_FEATURE_SW_REQUIRED_MASK,
	FIS_IDENTIFY_DIMM_FEATURE_SW_REQUIRED_MASK_INVALIDATE_BEFORE_BLOCK_READ,
	FIS_IDENTIFY_DIMM_FEATURE_SW_REQUIRED_MASK_READBACK_OF_BW_ADDRESS_REGISTER_REQUIRED_BEFORE_USE,
	FIS_IDENTIFY_DIMM_NUMBER_OF_BLOCK_WINDOWS,
	FIS_IDENTIFY_DIMM_OFFSET_OF_BLOCK_MODE_CONTROL_REGION,
	FIS_IDENTIFY_DIMM_RAW_CAPACITY,
	FIS_IDENTIFY_DIMM_MANUFACTURER,
	FIS_IDENTIFY_DIMM_SERIAL_NUMBER,
	FIS_IDENTIFY_DIMM_PART_NUMBER,
	FIS_IDENTIFY_DIMM_DIMM_SKU,
	FIS_IDENTIFY_DIMM_DIMM_SKU_MEMORY_MODE_ENABLED,
	FIS_IDENTIFY_DIMM_DIMM_SKU_STORAGE_MODE_ENABLED,
	FIS_IDENTIFY_DIMM_DIMM_SKU_APP_DIRECT_MODE_ENABLED,
	FIS_IDENTIFY_DIMM_DIMM_SKU_DIE_SPARING_CAPABLE,
	FIS_IDENTIFY_DIMM_DIMM_SKU_SOFT_PROGRAMMABLE_SKU,
	FIS_IDENTIFY_DIMM_DIMM_SKU_ENCRYPTION_ENABLED,
	FIS_IDENTIFY_DIMM_INTERFACE_FORMAT_CODE_EXTRA,
	FIS_IDENTIFY_DIMM_API_VER,
	FIS_IDENTIFY_DIMM_CHARACTERISTICS_CONTROLLER_TEMP_SHUTDOWN_THRESHOLD,
	FIS_IDENTIFY_DIMM_CHARACTERISTICS_MEDIA_TEMP_SHUTDOWN_THRESHOLD,
	FIS_IDENTIFY_DIMM_CHARACTERISTICS_THROTTLING_START_THRESHOLD,
	FIS_IDENTIFY_DIMM_CHARACTERISTICS_THROTTLING_STOP_THRESHOLD,
	FIS_GET_SECURITY_STATE_SECURITY_STATE,
	FIS_GET_SECURITY_STATE_SECURITY_STATE_ENABLED,
	FIS_GET_SECURITY_STATE_SECURITY_STATE_LOCKED,
	FIS_GET_SECURITY_STATE_SECURITY_STATE_FROZEN,
	FIS_GET_SECURITY_STATE_SECURITY_STATE_COUNT_EXPIRED,
	FIS_GET_SECURITY_STATE_SECURITY_STATE_NOT_SUPPORTED,
	FIS_GET_ALARM_THRESHOLD_ENABLE,
	FIS_GET_ALARM_THRESHOLD_ENABLE_SPARE_BLOCK,
	FIS_GET_ALARM_THRESHOLD_ENABLE_MEDIA_TEMP,
	FIS_GET_ALARM_THRESHOLD_ENABLE_CONTROLLER_TEMP,
	FIS_GET_ALARM_THRESHOLD_SPARE_BLOCK_THRESHOLD,
	FIS_GET_ALARM_THRESHOLD_MEDIA_TEMP_THRESHOLD,
	FIS_GET_ALARM_THRESHOLD_CONTROLLER_TEMP_THRESHOLD,
	FIS_POWER_MANAGEMENT_POLICY_ENABLE,
	FIS_POWER_MANAGEMENT_POLICY_PEAK_POWER_BUDGET,
	FIS_POWER_MANAGEMENT_POLICY_AVERAGE_POWER_BUDGET,
	FIS_POWER_MANAGEMENT_POLICY_MAX_POWER,
	FIS_DIE_SPARING_POLICY_ENABLE,
	FIS_DIE_SPARING_POLICY_AGGRESSIVENESS,
	FIS_DIE_SPARING_POLICY_SUPPORTED,
	FIS_DIE_SPARING_POLICY_SUPPORTED_RANK_0,
	FIS_DIE_SPARING_POLICY_SUPPORTED_RANK_1,
	FIS_DIE_SPARING_POLICY_SUPPORTED_RANK_2,
	FIS_DIE_SPARING_POLICY_SUPPORTED_RANK_3,
	FIS_ADDRESS_RANGE_SCRUB_ENABLE,
	FIS_ADDRESS_RANGE_SCRUB_DPA_START_ADDRESS,
	FIS_ADDRESS_RANGE_SCRUB_DPA_END_ADDRESS,
	FIS_ADDRESS_RANGE_SCRUB_DPA_CURRENT_ADDRESS,
	FIS_OPTIONAL_CONFIGURATION_DATA_POLICY_FIRST_FAST_REFRESH,
	FIS_OPTIONAL_CONFIGURATION_DATA_POLICY_VIRAL_POLICY_ENABLED,
	FIS_OPTIONAL_CONFIGURATION_DATA_POLICY_VIRAL_STATUS,
	FIS_PMON_REGISTERS_PMON_RETREIVE_MASK,
	FIS_PMON_REGISTERS_PMON_0_COUNTER,
	FIS_PMON_REGISTERS_PMON_0_CONTROL,
	FIS_PMON_REGISTERS_PMON_1_COUNTER,
	FIS_PMON_REGISTERS_PMON_1_CONTROL,
	FIS_PMON_REGISTERS_PMON_2_COUNTER,
	FIS_PMON_REGISTERS_PMON_2_CONTROL,
	FIS_PMON_REGISTERS_PMON_3_COUNTER,
	FIS_PMON_REGISTERS_PMON_3_CONTROL,
	FIS_PMON_REGISTERS_PMON_4_COUNTER,
	FIS_PMON_REGISTERS_PMON_4_CONTROL,
	FIS_PMON_REGISTERS_PMON_5_COUNTER,
	FIS_PMON_REGISTERS_PMON_5_CONTROL,
	FIS_PMON_REGISTERS_PMON_6_COUNTER,
	FIS_PMON_REGISTERS_PMON_6_CONTROL,
	FIS_PMON_REGISTERS_PMON_7_COUNTER,
	FIS_PMON_REGISTERS_PMON_7_CONTROL,
	FIS_PMON_REGISTERS_PMON_8_COUNTER,
	FIS_PMON_REGISTERS_PMON_8_CONTROL,
	FIS_PMON_REGISTERS_PMON_9_COUNTER,
	FIS_PMON_REGISTERS_PMON_9_CONTROL,
	FIS_PMON_REGISTERS_PMON_10_COUNTER,
	FIS_PMON_REGISTERS_PMON_10_CONTROL,
	FIS_PMON_REGISTERS_PMON_11_COUNTER,
	FIS_PMON_REGISTERS_PMON_11_CONTROL,
	FIS_PMON_REGISTERS_PMON_14_COUNTER,
	FIS_PMON_REGISTERS_PMON_14_CONTROL,
	FIS_SYSTEM_TIME_UNIX_TIME,
	FIS_PLATFORM_CONFIG_DATA_SIGNATURE,
	FIS_PLATFORM_CONFIG_DATA_LENGTH,
	FIS_PLATFORM_CONFIG_DATA_REVISION,
	FIS_PLATFORM_CONFIG_DATA_CHECKSUM,
	FIS_PLATFORM_CONFIG_DATA_OEM_ID,
	FIS_PLATFORM_CONFIG_DATA_OEM_TABLE_ID,
	FIS_PLATFORM_CONFIG_DATA_OEM_REVISION,
	FIS_PLATFORM_CONFIG_DATA_CREATOR_ID,
	FIS_PLATFORM_CONFIG_DATA_CREATOR_REVISION,
	FIS_PLATFORM_CONFIG_DATA_CURRENT_CONFIG_SIZE,
	FIS_PLATFORM_CONFIG_DATA_CURRENT_CONFIG_OFFSET,
	FIS_PLATFORM_CONFIG_DATA_INPUT_CONFIG_SIZE,
	FIS_PLATFORM_CONFIG_DATA_INPUT_CONFIG_OFFSET,
	FIS_PLATFORM_CONFIG_DATA_OUTPUT_CONFIG_SIZE,
	FIS_PLATFORM_CONFIG_DATA_OUTPUT_CONFIG_OFFSET,
	FIS_DIMM_PARTITION_INFO_VOLATILE_CAPACITY,
	FIS_DIMM_PARTITION_INFO_VOLATILE_START,
	FIS_DIMM_PARTITION_INFO_PM_CAPACITY,
	FIS_DIMM_PARTITION_INFO_PM_START,
	FIS_DIMM_PARTITION_INFO_RAW_CAPACITY,
	FIS_DIMM_PARTITION_INFO_ENABLED_CAPACITY,
	FIS_FW_DEBUG_LOG_LEVEL_LOG_LEVEL,
	FIS_FW_DEBUG_LOG_LEVEL_LOGS,
	FIS_FW_LOAD_FLAG_LOAD_FLAG,
	FIS_CONFIG_LOCKDOWN_LOCKED,
	FIS_DDRT_IO_INIT_INFO_DDRT_IO_INFO,
	FIS_DDRT_IO_INIT_INFO_DDRT_TRAINING_STATUS,
	FIS_GET_SUPPORTED_SKU_FEATURES_DIMM_SKU,
	FIS_ENABLE_DIMM_ENABLE,
	FIS_SMART_HEALTH_INFO_VALIDATION_FLAGS,
	FIS_SMART_HEALTH_INFO_VALIDATION_FLAGS_HEALTH_STATUS,
	FIS_SMART_HEALTH_INFO_VALIDATION_FLAGS_SPARE_BLOCKS,
	FIS_SMART_HEALTH_INFO_VALIDATION_FLAGS_PERCENT_USED,
	FIS_SMART_HEALTH_INFO_VALIDATION_FLAGS_MEDIA_TEMP,
	FIS_SMART_HEALTH_INFO_VALIDATION_FLAGS_CONTROLLER_TEMP,
	FIS_SMART_HEALTH_INFO_VALIDATION_FLAGS_UNSAFE_SHUTDOWN_COUNTER,
	FIS_SMART_HEALTH_INFO_VALIDATION_FLAGS_AIT_DRAM_STATUS,
	FIS_SMART_HEALTH_INFO_VALIDATION_FLAGS_ALARM_TRIPS,
	FIS_SMART_HEALTH_INFO_VALIDATION_FLAGS_LAST_SHUTDOWN_STATUS,
	FIS_SMART_HEALTH_INFO_VALIDATION_FLAGS_VENDOR_SPECIFIC_DATA_SIZE,
	FIS_SMART_HEALTH_INFO_HEALTH_STATUS,
	FIS_SMART_HEALTH_INFO_HEALTH_STATUS_NONCRITICAL,
	FIS_SMART_HEALTH_INFO_HEALTH_STATUS_CRITICAL,
	FIS_SMART_HEALTH_INFO_HEALTH_STATUS_FATAL,
	FIS_SMART_HEALTH_INFO_SPARE_BLOCKS,
	FIS_SMART_HEALTH_INFO_PERCENT_USED,
	FIS_SMART_HEALTH_INFO_ALARM_TRIPS,
	FIS_SMART_HEALTH_INFO_ALARM_TRIPS_SPARE_BLOCK_TRIP,
	FIS_SMART_HEALTH_INFO_ALARM_TRIPS_MEDIA_TEMPERATURE_TRIP,
	FIS_SMART_HEALTH_INFO_ALARM_TRIPS_CONTROLLER_TEMPERATURE_TRIP,
	FIS_SMART_HEALTH_INFO_MEDIA_TEMP,
	FIS_SMART_HEALTH_INFO_CONTROLLER_TEMP,
	FIS_SMART_HEALTH_INFO_UNSAFE_SHUTDOWN_COUNT,
	FIS_SMART_HEALTH_INFO_AIT_DRAM_STATUS,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS,
	FIS_SMART_HEALTH_INFO_VENDOR_SPECIFIC_DATA_SIZE,
	FIS_SMART_HEALTH_INFO_POWER_CYCLES,
	FIS_SMART_HEALTH_INFO_POWER_ON_TIME,
	FIS_SMART_HEALTH_INFO_UPTIME,
	FIS_SMART_HEALTH_INFO_UNSAFE_SHUTDOWNS,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_DETAILS,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_DETAILS_PM_ADR_COMMAND_RECEIVED,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_DETAILS_PM_S3_RECEIVED,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_DETAILS_PM_S5_RECEIVED,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_DETAILS_DDRT_POWER_FAIL_COMMAND_RECEIVED,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_DETAILS_PMIC_12V_POWER_FAIL,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_DETAILS_PM_WARM_RESET_RECEIVED,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_DETAILS_THERMAL_SHUTDOWN_RECEIVED,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_DETAILS_FLUSH_COMPLETE,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_TIME,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_EXTENDED_DETAILS,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_EXTENDED_DETAILS_VIRAL_INTERRUPT_RECEIVED,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_EXTENDED_DETAILS_SURPRISE_CLOCK_STOP_INTERRUPT_RECEIVED,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_EXTENDED_DETAILS_WRITE_DATA_FLUSH_COMPLETE,
	FIS_SMART_HEALTH_INFO_LAST_SHUTDOWN_STATUS_EXTENDED_DETAILS_S4_POWER_STATE_RECEIVED,
	FIS_SMART_HEALTH_INFO_MEDIA_ERROR_INJECTIONS,
	FIS_SMART_HEALTH_INFO_NON_MEDIA_ERROR_INJECTIONS,
	FIS_FIRMWARE_IMAGE_INFO_FIRMWARE_REVISION,
	FIS_FIRMWARE_IMAGE_INFO_FIRMWARE_TYPE,
	FIS_FIRMWARE_IMAGE_INFO_STAGED_FW_REVISION,
	FIS_FIRMWARE_IMAGE_INFO_LAST_FW_UPDATE_STATUS,
	FIS_FIRMWARE_IMAGE_INFO_COMMIT_ID,
	FIS_FIRMWARE_IMAGE_INFO_BUILD_CONFIGURATION,
	FIS_FIRMWARE_DEBUG_LOG_LOG_SIZE,
	FIS_MEMORY_INFO_PAGE_0_MEDIA_READS,
	FIS_MEMORY_INFO_PAGE_0_MEDIA_WRITES,
	FIS_MEMORY_INFO_PAGE_0_READ_REQUESTS,
	FIS_MEMORY_INFO_PAGE_0_WRITE_REQUESTS,
	FIS_MEMORY_INFO_PAGE_0_BLOCK_READ_REQUESTS,
	FIS_MEMORY_INFO_PAGE_0_BLOCK_WRITE_REQUESTS,
	FIS_MEMORY_INFO_PAGE_1_TOTAL_MEDIA_READS,
	FIS_MEMORY_INFO_PAGE_1_TOTAL_MEDIA_WRITES,
	FIS_MEMORY_INFO_PAGE_1_TOTAL_READ_REQUESTS,
	FIS_MEMORY_INFO_PAGE_1_TOTAL_WRITE_REQUESTS,
	FIS_MEMORY_INFO_PAGE_1_TOTAL_BLOCK_READ_REQUESTS,
	FIS_MEMORY_INFO_PAGE_1_TOTAL_BLOCK_WRITE_REQUESTS,
	FIS_MEMORY_INFO_PAGE_3_ERROR_INJECTION_STATUS,
	FIS_MEMORY_INFO_PAGE_3_ERROR_INJECTION_STATUS_ERROR_INJECTION_ENABLED,
	FIS_MEMORY_INFO_PAGE_3_ERROR_INJECTION_STATUS_MEDIA_TEMPERATURE_INJECTION_ENABLED,
	FIS_MEMORY_INFO_PAGE_3_ERROR_INJECTION_STATUS_SOFTWARE_TRIGGERS_ENABLED,
	FIS_MEMORY_INFO_PAGE_3_POISON_ERROR_INJECTIONS_COUNTER,
	FIS_MEMORY_INFO_PAGE_3_POISON_ERROR_CLEAR_COUNTER,
	FIS_MEMORY_INFO_PAGE_3_MEDIA_TEMPERATURE_INJECTIONS_COUNTER,
	FIS_MEMORY_INFO_PAGE_3_SOFTWARE_TRIGGERS_COUNTER,
	FIS_LONG_OPERATION_STATUS_COMMAND,
	FIS_LONG_OPERATION_STATUS_PERCENT_COMPLETE,
	FIS_LONG_OPERATION_STATUS_ESTIMATE_TIME_TO_COMPLETION,
	FIS_LONG_OPERATION_STATUS_STATUS_CODE,
	FIS_LONG_OPERATION_STATUS_COMMAND_SPECIFIC_RETURN_DATA,
	FIS_BSR_MAJOR_CHECKPOINT,
	FIS_BSR_MINOR_CHECKPOINT,
	FIS_BSR_REST1,
	FIS_BSR_REST1_MEDIA_READY_1,
	FIS_BSR_REST1_MEDIA_READY_2,
	FIS_BSR_REST1_DDRT_IO_INIT_COMPLETE,
	FIS_BSR_REST1_PCR_LOCK,
	FIS_BSR_REST1_MAILBOX_READY,
	FIS_BSR_REST1_WATCH_DOG_STATUS,
	FIS_BSR_REST1_FIRST_FAST_REFRESH_COMPLETE,
	FIS_BSR_REST1_CREDIT_READY,
	FIS_BSR_REST1_MEDIA_DISABLED,
	FIS_BSR_REST1_OPT_IN_ENABLED,
	FIS_BSR_REST1_OPT_IN_WAS_ENABLED,
	FIS_BSR_REST1_ASSERTION,
	FIS_BSR_REST1_MI_STALL,
	FIS_BSR_REST1_AIT_DRAM_READY,
	FIS_BSR_REST2,
	IXP_PROP_KEY_MAX
};

#ifdef __cplusplus
}
#endif

#endif /* SRC_IXP_TYPES_H_ */